﻿using AspNetCoreLecture.DAL.EFCore.Entities;
using Microsoft.EntityFrameworkCore;

namespace AspNetCoreLecture.DAL.EFCore
{
    public class SchoolDbContext : DbContext
    {
        public SchoolDbContext() { }

        public SchoolDbContext(DbContextOptions<SchoolDbContext> options) : base(options) { }

        public DbSet<Student> Students { get; set; }
        public DbSet<Course> Courses { get; set; }
        public DbSet<Grade> Grades { get; set; }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            base.OnModelCreating(modelBuilder);

            modelBuilder.Entity<Student>().HasData(
                new Student()
                {
                    Id = 1,
                    FirstName = "Billy",
                    LastName = "Herrington"
                },
                new Student()
                {
                    Id = 2,
                    FirstName = "Peter",
                    LastName = "Soily"
                },
                new Student()
                {
                    Id = 3,
                    FirstName = "Grzegorz",
                    LastName = "Brzęczyszczykiewicz"
                });

            modelBuilder.Entity<Course>().HasData(
                new Course()
                {
                    Id = 1,
                    Name = "Introduction to BDSM (PB161)"
                },
                new Course()
                {
                    Id = 2,
                    Name = "FFF FF FE (IB002)"
                });

            modelBuilder.Entity<Grade>().HasData(
                new Grade()
                {
                    Id = 1,
                    CourseId = 1,
                    StudentId = 1,
                    GradeLetter = "A",
                },
                new Grade()
                {
                    Id = 2,
                    CourseId = 2,
                    StudentId = 2,
                    GradeLetter = "A",
                },
                new Grade()
                {
                    Id = 3,
                    CourseId = 2,
                    StudentId = 3,
                    GradeLetter = "F",
                });
        }
    }
}
