﻿using AspNetCoreLecture.DAL.EFCore.Entities;
using AspNetCoreLecture.Data.Dto;
using Mapster;
using Microsoft.EntityFrameworkCore;

namespace AspNetCoreLecture.Data.Repository
{
    public class RepositoryBase<TEntity, TDto> : IRepository<TDto> 
        where TDto : class, IEntityDto
        where TEntity : class, IEntity
    {
        private DbContext _dbContext { get; }

        public RepositoryBase(DbContext dbContext)
        {
            _dbContext = dbContext;
        }

        public void Create(TDto record)
        {
            var entity = record.Adapt<TEntity>();
            _dbContext.Set<TEntity>().Add(entity);
            _dbContext.SaveChanges();
            record.Id = entity.Id;
        }

        public void Update(TDto record)
        {
            var entity = record.Adapt<TEntity>();
            var originalEntity = _dbContext.Find<TEntity>(record.Id);
            _dbContext.Entry(originalEntity).CurrentValues.SetValues(entity);
            _dbContext.SaveChanges();
        }

        public void Delete(TDto record)
        {
            var entity = _dbContext.Find<TEntity>(record.Id);
            if (entity is null)
                return;
            _dbContext.Entry(entity).State = EntityState.Deleted;
            _dbContext.SaveChanges();
        }

        public TDto Get(int id)
        {
            return _dbContext.Set<TEntity>().AsNoTracking().Where(x => x.Id == id).FirstOrDefault()?.Adapt<TDto>();
        }

        public IEnumerable<TDto> Get(IEnumerable<int> ids)
        {
            return _dbContext.Set<TEntity>().AsNoTracking().Where(x => ids.Contains(x.Id)).AsEnumerable().Select(x => x.Adapt<TDto>()).ToArray();
        }

        public IEnumerable<TDto> GetAll()
        {
            return _dbContext.Set<TEntity>().AsNoTracking().AsEnumerable().Select(x => x.Adapt<TDto>()).ToArray();
        }
    }
}
