﻿using AspNetCoreLecture.Data.Repository;
using AspNetCoreLecture.Models;
using Mapster;
using Microsoft.AspNetCore.Mvc;

namespace AspNetCoreLecture.Controllers
{
    public class StudentController : Controller
    {
        private readonly IStudentRepository _studentRepository;

        public StudentController(IStudentRepository studentRepository)
        {
            _studentRepository = studentRepository;
        }

        public IActionResult Index()
        {
            var model = new StudentIndexViewModel()
            {
                Students = _studentRepository.GetAll()
            };

            return View(model);
        }

        public IActionResult Add()
        {
            return View(new StudentUpsertViewModel());
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Add(StudentUpsertViewModel model)
        {
            if (!ModelState.IsValid)
            {
                return View(model);
            }

            var dto = model.ToDto();
            _studentRepository.Create(dto);

            return RedirectToAction(nameof(Index));
        }

        public IActionResult Edit(int id)
        {
            var dto = _studentRepository.Get(id);
            if (dto == null)
            {
                return NotFound();
            }

            // Just a lazy way to map the DTO 1:1 to a view model
            var model = dto.Adapt<StudentUpsertViewModel>();
            return View(model);
        }

        // We are not in a REST controller, and therefore can afford to make the delete logic a POST method. And also ASP.NET HTML helpers do not have the FormMethod.Delete member :D
        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Delete(int id)
        {
            var dto = _studentRepository.Get(id);
            if (dto == null)
            {
                return NotFound();
            }

            _studentRepository.Delete(dto);

            return RedirectToAction(nameof(Index));
        }
    }
}
