using AspNetCoreLecture.Data.Repository;
using AspNetCoreLecture.WebApi.Models;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;

namespace AspNetCoreLecture.WebApi.Controllers
{
    [ApiController]
    // This means that the route prefix pro this controller is "Student" -> parsed from "StudentController"
    [Route("[controller]")]
    public class StudentController : ControllerBase
    {
        private readonly IStudentRepository studentRepository;

        public StudentController(IStudentRepository studentRepository)
        {
            this.studentRepository = studentRepository;
        }

        // HTTP GET is implicit in ASP.NET Core, but not in Swagger...
        [HttpGet]
        [Route("Status")]
        public string StatusCheck()
        {
            return "OK";
        }

        [HttpGet]
        [Route("All")]
        public IEnumerable<StudentModel> GetAll()
        {
            return studentRepository.GetAll()
                .Select(x => new StudentModel()
                {
                    Id = x.Id,
                    Name = x.FirstName,
                    Surname = x.LastName
                });
        }

        [HttpGet]
        [Route("AllAsync")]
        public async Task<IEnumerable<StudentModel>> GetAllAsync()
        {
            // Let's simulate an async operation
            await Task.Delay(TimeSpan.FromSeconds(3));

            // Now let us suppose that something goes wrong...
            if (Random.Shared.NextDouble() > 0.5d)
            {
                // What now? We need to return an error HTTP code...
            }

            return studentRepository.GetAll()
                .Select(x => new StudentModel()
                {
                    Id = x.Id,
                    Name = x.FirstName,
                    Surname = x.LastName
                });
        }

        [HttpPost]
        public IActionResult Insert(StudentModel student)
        {
            // Let's see what ASP.NET Core can bind automatically
            return Ok();
        }
    }
}