using AspNetCoreLecture.DAL.EFCore;
using AspNetCoreLecture.Data.Dto;
using AspNetCoreLecture.Data.Repository;
using Microsoft.EntityFrameworkCore;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen();
builder.Services.AddDbContext<SchoolDbContext>(builder => builder.UseSqlite("Data Source=School.db;Cache=Shared"));
builder.Services.AddTransient<DbContext>(x => x.GetRequiredService<SchoolDbContext>());
builder.Services.AddTransient<IRepository<CourseDto>, CourseRepository>();
builder.Services.AddTransient<ICourseRepository, CourseRepository>();
builder.Services.AddTransient<IRepository<GradeDto>, GradeRepository>();
builder.Services.AddTransient<IGradeRepository, GradeRepository>();
builder.Services.AddTransient<IRepository<StudentDto>, StudentRepository>();
builder.Services.AddTransient<IStudentRepository, StudentRepository>();

var app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI();
}

app.UseHttpsRedirection();

app.UseAuthorization();

app.MapControllers();

// This creates the database if it does not exist yet. Incompatible with migrations, but a quick way to set up a working database schema.
using (var scope = app.Services.CreateScope())
{
    await scope.ServiceProvider.GetRequiredService<SchoolDbContext>().Database.EnsureCreatedAsync().ConfigureAwait(false);
}

app.Run();
